/*
 * Copyright (c) 2007, Intalio Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Intalio Inc. - initial API and implementation
 */
package org.eclipse.stp.bpmn.diagram.edit.parts;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PrecisionPoint;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.draw2d.text.AbstractFlowBorder;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.FeatureMap;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.LayoutEditPolicy;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gmf.runtime.diagram.ui.editparts.IGraphicalEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editparts.ShapeNodeEditPart;
import org.eclipse.gmf.runtime.diagram.ui.editpolicies.EditPolicyRoles;
import org.eclipse.gmf.runtime.diagram.ui.requests.CreateUnspecifiedTypeConnectionRequest;
import org.eclipse.gmf.runtime.draw2d.ui.figures.ConstrainedToolbarLayout;
import org.eclipse.gmf.runtime.draw2d.ui.figures.PolylineConnectionEx;
import org.eclipse.gmf.runtime.draw2d.ui.figures.WrapLabel;
import org.eclipse.gmf.runtime.emf.type.core.MetamodelType;
import org.eclipse.gmf.runtime.gef.ui.figures.NodeFigure;
import org.eclipse.gmf.runtime.notation.Bounds;
import org.eclipse.gmf.runtime.notation.Location;
import org.eclipse.gmf.runtime.notation.Node;
import org.eclipse.gmf.runtime.notation.NotationFactory;
import org.eclipse.gmf.runtime.notation.NotationPackage;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.stp.bpmn.Activity;
import org.eclipse.stp.bpmn.ActivityType;
import org.eclipse.stp.bpmn.BpmnPackage;
import org.eclipse.stp.bpmn.diagram.edit.policies.ActivityCanonicalEditPolicy;
import org.eclipse.stp.bpmn.diagram.edit.policies.ActivityGraphicalNodeEditPolicy;
import org.eclipse.stp.bpmn.diagram.edit.policies.ActivityItemSemanticEditPolicy;
import org.eclipse.stp.bpmn.diagram.part.BpmnVisualIDRegistry;
import org.eclipse.stp.bpmn.diagram.providers.BpmnElementTypes;
import org.eclipse.stp.bpmn.figures.activities.ActivityDiamondFigure;
import org.eclipse.stp.bpmn.figures.activities.ActivityNodeFigure;
import org.eclipse.stp.bpmn.figures.activities.ActivityOvalFigure;
import org.eclipse.stp.bpmn.figures.connectionanchors.IModelAwareAnchor;
import org.eclipse.stp.bpmn.figures.connectionanchors.WrapperNodeFigureEx;
import org.eclipse.stp.bpmn.figures.connectionanchors.impl.ConnectionAnchorFactory;
import org.eclipse.stp.bpmn.policies.BpmnDragDropEditPolicy;
import org.eclipse.stp.bpmn.policies.ConnectionHandleEditPolicyEx;
import org.eclipse.stp.bpmn.policies.OpenFileEditPolicy;
import org.eclipse.stp.bpmn.policies.ResizableActivityEditPolicy;
import org.eclipse.stp.bpmn.tools.EdgeConnectionValidator;
import org.eclipse.stp.bpmn.tools.MessageConnectionValidator;
import org.eclipse.stp.bpmn.tools.TaskDragEditPartsTrackerEx;

/**
 * @generated
 */
public class ActivityEditPart extends ShapeNodeEditPart {
    // private Point targetFigureLocation = new Point(-300, -300);

    private static final int SHAPE_RECTANGLE = 0;

    private static final int SHAPE_DIAMOND = 1;

    private static final int SHAPE_CIRCLE = 2;

    /**
     * @notgenerated
     */
    public static final int EVENT_FIGURE_SIZE = 30;

    /**
     * @notgenerated
     */
    public static final int GATEWAY_FIGURE_SIZE = 50;

    /**
     * @notgenerated
     */
    public static final Dimension ACTIVITY_FIGURE_SIZE = new Dimension(111, 61);

    /**
     * @generated
     */
    public static final int VISUAL_ID = 2001;

    /**
     * @notgenerated
     */
    private int handlePosition;

    /**
     * @notgenerated
     */
    protected boolean isChildAdded = false;

    /**
     * @generated
     */
    protected IFigure contentPane;

    /**
     * @generated
     */
    protected IFigure primaryShape;

    /**
     * Structure of figure: WrapperNodeFigure is a node plate figure that wraps
     * Oval, Diamond or rectanle, that holds primary shape. Wrapped shape can be
     * chaged when activity type is changed.
     * 
     * @notgenerated
     */
    private NodeFigure wrappedFigure;

    /**
     * @generated
     */
    public ActivityEditPart(View view) {
        super(view);
    }

    /**
     * @generated
     */
    protected void createDefaultEditPoliciesGen() {
        super.createDefaultEditPolicies();
        installEditPolicy(EditPolicyRoles.SEMANTIC_ROLE,
                new ActivityItemSemanticEditPolicy());
        installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
                new ActivityGraphicalNodeEditPolicy());
        installEditPolicy(EditPolicyRoles.CANONICAL_ROLE,
                new ActivityCanonicalEditPolicy());
        installEditPolicy(EditPolicy.LAYOUT_ROLE, createLayoutEditPolicy());
    }

    /**
     * @generated NOT
     */
    protected void createDefaultEditPolicies() {
        createDefaultEditPoliciesGen();

        // replace ConnectionHandleEditPolicy with our own
        removeEditPolicy(EditPolicyRoles.CONNECTION_HANDLES_ROLE);
        installEditPolicy(EditPolicyRoles.CONNECTION_HANDLES_ROLE,
                new ConnectionHandleEditPolicyEx());
        // adding default drag and drop edit policy
        installEditPolicy(EditPolicyRoles.DRAG_DROP_ROLE,
                new BpmnDragDropEditPolicy(this));
        // adding an open edit policy
        installEditPolicy(EditPolicyRoles.OPEN_ROLE,
        		new OpenFileEditPolicy());
    }

    /**
     * @generated
     */
    protected LayoutEditPolicy createLayoutEditPolicy() {
        LayoutEditPolicy lep = new LayoutEditPolicy() {

            protected EditPolicy createChildEditPolicy(EditPart child) {
                EditPolicy result = 
                    child.getEditPolicy(EditPolicy.PRIMARY_DRAG_ROLE);
                if (result == null) {
                	result = getPrimaryDragEditPolicy();
                	if (result != null) {
                		return result;
                	}
                    result = new NonResizableEditPolicy();
                }
                return result;
            }

            protected Command getMoveChildrenCommand(Request request) {
                return null;
            }

            protected Command getCreateCommand(CreateRequest request) {
                return null;
            }
        };
        return lep;
    }

    @Override
    public void installEditPolicy(Object key, EditPolicy editPolicy) {
        super.installEditPolicy(key, editPolicy);
    }

    /**
     * @generated
     */
    protected IFigure createNodeShapeGen() {
        ActivityFigure figure = new ActivityFigure();
        return primaryShape = figure;
    }

    /**
     * @notgenerated
     */
    protected IFigure createNodeShape() {
        ActivityFigure figure = (ActivityFigure) createNodeShapeGen();
        Activity activity = (Activity) getPrimaryView().getElement();
        setActivityTypeAndLabelAndLayout(figure, activity);
        figure.setLooping(activity.isLooping());
        return figure;
    }

    /**
     * 
     * @param activityFigure
     * @param activity
     * @return true if refreshVisual is recommended (there was a change)
     */
    private boolean setActivityTypeAndLabelAndLayout(
            ActivityFigure activityFigure, Activity activity) {
        activityFigure.setActivityType(activity.getActivityType().getName());
        boolean res = false;
        if (activity.getName() == null) {
            WrapLabel wl = activityFigure.getFigureActivityNameFigure();
            if (activity.getActivityType().equals(ActivityType.TASK_LITERAL)) {
                wl.setTextWrap(true);
                if (!"Task".equals(wl.getText())) {
                    wl.setText("Task");
                }
            }
            res = true;
        }

        if (activity.getActivityType().equals(ActivityType.TASK_LITERAL)) {
            if (!(activityFigure.getLayoutManager() instanceof StackLayout)) {
                StackLayout layout = new StackLayout();
                activityFigure.setLayoutManager(layout);
                res = true;
            }
        } else {
            if (!(activityFigure.getLayoutManager() instanceof ConstrainedToolbarLayout)) {
                ConstrainedToolbarLayout layout = new ConstrainedToolbarLayout();
                layout.setSpacing(getMapMode().DPtoLP(5));
                activityFigure.setLayoutManager(layout);
                res = true;
            }
        }

        return res;
    }

    /**
     * @generated
     */
    public ActivityFigure getPrimaryShape() {
        return (ActivityFigure) primaryShape;
    }

    /**
     * @generated
     */
    protected boolean addFixedChild(EditPart childEditPart) {
        if (childEditPart instanceof ActivityNameEditPart) {
            ((ActivityNameEditPart) childEditPart).setLabel(getPrimaryShape()
                    .getFigureActivityNameFigure());
            return true;
        }
        return false;
    }

    /**
     * @generated
     */
    protected boolean removeFixedChild(EditPart childEditPart) {
        return false;
    }

    /**
     * @notgenerated
     */
    protected NodeFigure createNodePlate() {

        // final Activity2EditPart zis = this;

        return new WrapperNodeFigureEx(ConnectionAnchorFactory.INSTANCE,
                createWrappedFigure()) {
            @Override
            /**
             * Exclude activity name label from handle bounds
             */
            public Rectangle getHandleBounds() {
                Rectangle rectangle = super.getHandleBounds();
                Activity activity = (Activity) resolveSemanticElement();
                if (activity == null || activity.getActivityType() == null || 
                		activity.getActivityType().equals(ActivityType.TASK_LITERAL)) {
                    return rectangle;
                }

                IGraphicalEditPart activityNameEditPart = getChildBySemanticHint(BpmnVisualIDRegistry
                        .getType(org.eclipse.stp.bpmn.diagram.edit.parts.ActivityNameEditPart.VISUAL_ID));
                int nameFigureHeight = activityNameEditPart == null 
                		|| activityNameEditPart.getFigure() == null ? 
                		0 : activityNameEditPart.getFigure().getSize().height;

                return new Rectangle(rectangle.x, rectangle.y, rectangle.width,
                        rectangle.height - nameFigureHeight);

            };

            @Override
            public void setBounds(Rectangle rect) {
                int x = bounds.x, y = bounds.y;

                boolean resize = (rect.width != bounds.width)
                        || (rect.height != bounds.height), translate = (rect.x != x)
                        || (rect.y != y);

                if ((resize || translate) && isVisible())
                    erase();
                if (translate) {
                    int dx = rect.x - x;
                    int dy = rect.y - y;
                    primTranslate(dx, dy);
                }

                bounds.width = rect.width;
                bounds.height = rect.height;

                if (translate || resize) {
                    if (resize || isChildAdded && translate) {
                        invalidate();
                    }
                    fireFigureMoved();
                    repaint();
                }

            }
        };
    }

    /**
     * @notgenerated
     * @return
     */
    private NodeFigure createWrappedFigure() {
        Activity activity = (Activity) getPrimaryView().getElement();
        final int activityType = activity.getActivityType().getValue();

        int shapeType = getShapeType(activityType);
        if (shapeType == SHAPE_RECTANGLE) {
            int width = getMapMode().DPtoLP(ACTIVITY_FIGURE_SIZE.width);
            int height = getMapMode().DPtoLP(ACTIVITY_FIGURE_SIZE.height);
            wrappedFigure = new ActivityNodeFigure(width, height);
        } else if (shapeType == SHAPE_DIAMOND) {
        	int width = getMapMode().DPtoLP(GATEWAY_FIGURE_SIZE);
            int height = getMapMode().DPtoLP(GATEWAY_FIGURE_SIZE);
            wrappedFigure = new ActivityNodeFigure(width, height);
        } else {
        	int width = getMapMode().DPtoLP(EVENT_FIGURE_SIZE);
            int height = getMapMode().DPtoLP(EVENT_FIGURE_SIZE);
            wrappedFigure = new ActivityNodeFigure(width, height);
        }
        wrappedFigure.setLayoutManager(new StackLayout());
        return wrappedFigure;
    }

    /**
     * @notgenerated
     */
    private void buildFigure(IFigure container, IFigure shape,
            WrapLabel wrapLabel, int size) {
        int resultSize = getMapMode().DPtoLP(size);
        container.setSize(resultSize, resultSize);

        wrappedFigure.setLayoutManager(new BorderLayout());
        container.setLayoutManager(new StackLayout());
        container.add(shape);

        wrappedFigure.add(container, BorderLayout.CENTER);
        wrappedFigure.add(wrapLabel, BorderLayout.BOTTOM);
        wrapLabel.setTextWrap(true);
        wrapLabel.setLabelAlignment(PositionConstants.CENTER);
        ((ActivityFigure) shape).setFigureActivityNameFigure(wrapLabel);

        contentPane = setupContentPane(wrappedFigure);
    }




    /**
     * Creates figure for this edit part.
     * 
     * Body of this method does not depend on settings in generation model so
     * you may safely remove <i>generated</i> tag and modify it.
     * 
     * @notgenerated
     */
    protected NodeFigure createNodeFigure() {
        NodeFigure figure = createNodePlate();
        IFigure shape = createNodeShape();

        ActivityType activityType = ((Activity) getPrimaryView().getElement())
                .getActivityType();
        boolean gateway = ActivityType.VALUES_GATEWAYS.contains(activityType);
        boolean event = ActivityType.VALUES_EVENTS.contains(activityType)
                || ActivityType.VALUES_EVENTS_INTERMEDIATE
                        .contains(activityType);

        if (gateway) {
            int size = getMapMode().DPtoLP(GATEWAY_FIGURE_SIZE);
            IFigure diamondFigure = new ActivityDiamondFigure(new Dimension(
                    size, size));
            WrapLabel label = new WrapLabel();
            buildFigure(diamondFigure, shape, label, GATEWAY_FIGURE_SIZE);
        } else if (event) {
            ActivityOvalFigure ovalFigure = new ActivityOvalFigure();
            WrapLabel label = new WrapLabel();
            buildFigure(ovalFigure, shape, label, EVENT_FIGURE_SIZE);
        } else {
            wrappedFigure.add(shape);
            contentPane = setupContentPane(shape);
        }
        return figure;
    }

    /**
     * Default implementation treats passed figure as content pane. Respects
     * layout one may have set for generated figure.
     * 
     * @param nodeShape
     *            instance of generated figure class
     * @generated
     */
    protected IFigure setupContentPaneGen(IFigure nodeShape) {
        if (nodeShape.getLayoutManager() == null) {
            ConstrainedToolbarLayout layout = new ConstrainedToolbarLayout();
            layout.setSpacing(getMapMode().DPtoLP(5));
            nodeShape.setLayoutManager(layout);
        }
        return nodeShape; // use nodeShape itself as contentPane
    }

    /**
     * @notgenerated
     */
    protected IFigure setupContentPane(IFigure nodeShape) {
        if (nodeShape.getLayoutManager() == null) {
            EObject element = ((Node) getModel()).getElement();
            int activityType = ((Activity) element).getActivityType()
                    .getValue();
            if (activityType == ActivityType.TASK) {
                StackLayout layout = new StackLayout();
                nodeShape.setLayoutManager(layout);
            } else {
                return setupContentPaneGen(nodeShape);
            }
        }
        return nodeShape; // use nodeShape itself as contentPane
    }

    /**
     * @generated
     */
    public IFigure getContentPane() {
        if (contentPane != null) {
            return contentPane;
        }
        return super.getContentPane();
    }

    /**
     * @generated
     */
    public EditPart getPrimaryChildEditPart() {
        return getChildBySemanticHint(BpmnVisualIDRegistry
                .getType(ActivityNameEditPart.VISUAL_ID));
    }

    /**
     * @generated
     */
    protected void addChildVisual(EditPart childEditPart, int index) {
        if (addFixedChild(childEditPart)) {
            return;
        }
        super.addChildVisual(childEditPart, -1);
    }

    /**
     * @generated
     */
    protected void removeChildVisual(EditPart childEditPart) {
        if (removeFixedChild(childEditPart)) {
            return;
        }
        super.removeChildVisual(childEditPart);
    }

    /**
     * 
     * @return true if activity type is event or gateway, false otherwise
     * @notgenerated
     */
    private boolean isEventOrGateway() {
        ActivityType activityType = ((Activity) getPrimaryView().getElement())
                .getActivityType();
        boolean gateway = ActivityType.VALUES_GATEWAYS.contains(activityType);
        boolean event = ActivityType.VALUES_EVENTS.contains(activityType)
                || ActivityType.VALUES_EVENTS_INTERMEDIATE
                        .contains(activityType);

        return gateway || event;
    }

    /**
     * Synchronizes the shape with the activityType
     * 
     * @notgenerated
     * @see org.eclipse.gmf.runtime.diagram.ui.editparts.GraphicalEditPart
     *      #handlePropertyChangeEvent(java.beans.PropertyChangeEvent)
     */
    protected void handleNotificationEvent(Notification notification) {
        if (notification.getEventType() == Notification.SET) {
            if (BpmnPackage.eINSTANCE.getActivity_ActivityType().equals(
                    notification.getFeature())) {
                // figure = null;
                ActivityType oldActivityType = (ActivityType) notification
                        .getOldValue();
                ActivityType newActivityType = (ActivityType) notification
                        .getNewValue();
              //  maybeChangeFigure(oldActivityType, newActivityType);
//                getPrimaryShape().setActivityType(newActivityType.getLiteral());
                // refreshVisuals();
//                handleMajorSemanticChange();
//                return;
//                Bounds bounds = NotationFactory.eINSTANCE.createBounds();
//                bounds.setX(((Location) ((Node) getNotationView()).getLayoutConstraint()).getX());
//                bounds.setY(((Location) ((Node) getNotationView()).getLayoutConstraint()).getY());
//                if (ActivityType.VALUES_GATEWAYS.contains(newActivityType)) {
//                	bounds.setHeight(GATEWAY_FIGURE_SIZE);
//                	bounds.setWidth(GATEWAY_FIGURE_SIZE);
//                } else if (ActivityType.VALUES_EVENTS.contains(newActivityType)) {
//                	bounds.setHeight(EVENT_FIGURE_SIZE);
//                	bounds.setWidth(EVENT_FIGURE_SIZE);
//                } else if (newActivityType.getValue() == ActivityType.TASK) {
//                	bounds.setHeight(ACTIVITY_FIGURE_SIZE.height);
//                	bounds.setWidth(ACTIVITY_FIGURE_SIZE.width);
//                }
//                ((Node) getNotationView()).setLayoutConstraint(bounds);
                handleMajorSemanticChange();
              return;
            } else if (BpmnPackage.eINSTANCE.getActivity_Looping().equals(
                    notification.getFeature())) {
                // looping changed
                getPrimaryShape().setLooping(notification.getNewBooleanValue());
            } else if (NotationPackage.eINSTANCE.getSize_Width().equals(
                    notification.getFeature())
                    || NotationPackage.eINSTANCE.getSize_Height().equals(
                            notification.getFeature())) {
                setChildAdded(true);
            } else if (BpmnPackage.eINSTANCE.getNamedBpmnObject_Name().equals(
                    notification.getFeature())
                    && isEventOrGateway()) {
//                changeActivitySize(notification.getOldStringValue(),
//                        notification.getNewStringValue());
            }
            Activity activity = (Activity) getPrimaryView().getElement();
            if (setActivityTypeAndLabelAndLayout(getPrimaryShape(), activity)) {
                refreshVisuals();
            }
        }

        super.handleNotificationEvent(notification);
    }

    @Override
	public void refreshSourceConnections() {
        super.refreshSourceConnections();
        updateAnchors(true);
    }

    @Override
	public void refreshTargetConnections() {
        super.refreshTargetConnections();
        updateAnchors(false);
    }

    /**
     * Computes the indexes on the anchors so that they can compute their position.
     * 
     * @notgenerated
     */
    private void updateAnchors(boolean sourceOnly) {
        Map<EObject, ConnectionEditPart> connIndex =
            new HashMap<EObject, ConnectionEditPart>();
        
        for (Object object : getSourceConnections()) {
            ConnectionEditPart connEditPart = (ConnectionEditPart) object;
            View model = (View) connEditPart.getModel();
            connIndex.put(model.getElement(), connEditPart);
        }
        for (Object object : getTargetConnections()) {
            ConnectionEditPart connEditPart = (ConnectionEditPart) object;
            View model = (View) connEditPart.getModel();
            connIndex.put(model.getElement(), connEditPart);
        }
        FeatureMap messages =
            ((Activity) (((Node) getModel()).getElement()))
                .getOrderedMessages();
        int totalLength = messages.size();
        int i = 0;
        for (Iterator it = messages.iterator(); it.hasNext(); ) {
            FeatureMap.Entry msg = (FeatureMap.Entry)it.next();
            switch (msg.getEStructuralFeature().getFeatureID()) {
            case BpmnPackage.ACTIVITY__OUTGOING_MESSAGES:
                //sourceMsgs.put(msg.getValue(), i);
                setAnchorIndex(connIndex, (EObject)msg.getValue(),
                        i, totalLength, true,
                        String.valueOf(MessagingEdgeEditPart.VISUAL_ID));
                i++;
                break;
            case BpmnPackage.ACTIVITY__INCOMING_MESSAGES:
                //targetMsgs.put(msg.getValue(), i);
                setAnchorIndex(connIndex, (EObject)msg.getValue(),
                        i, totalLength, false,
                        String.valueOf(MessagingEdgeEditPart.VISUAL_ID));
                i++;
                break;
            default: throw new IllegalStateException("Corrupted model?");
            }
        }
        //same for outgoing sequences:
        if (sourceOnly) {
            EList outEdges = ((Activity) (((Node) getModel()).getElement()))
                .getOutgoingEdges();
            int ind = 0;
            totalLength = outEdges.size();
            for (Object edge : outEdges) {
                setAnchorIndex(connIndex, (EObject)edge, i, totalLength, true,
                        String.valueOf(SequenceEdgeEditPart.VISUAL_ID));
                ind++;
            }
        } else {
            //same for incoming sequences:
            EList inEdges = ((Activity) (((Node) getModel()).getElement()))
                .getIncomingEdges();
            int ind = 0;
            totalLength = inEdges.size();
            for (Object edge : inEdges) {
                setAnchorIndex(connIndex, (EObject)edge, i, totalLength, false,
                        String.valueOf(SequenceEdgeEditPart.VISUAL_ID));
                ind++;
            }
        }
    }
    
    private void setAnchorIndex(Map<EObject, ConnectionEditPart> connIndex, 
            EObject msgOrSequence, int index, int totalLength, boolean isSource,
            String edgeType) {
        ConnectionEditPart conn = connIndex.get(msgOrSequence);
        if (conn == null) {
            //this is usually the sign of a corrupted model :(
            System.err.println("Could not find a ConnectionEditPart for " +
                    msgOrSequence);
        } else {
            PolylineConnectionEx connFigure = 
                (PolylineConnectionEx) conn.getFigure();
            ConnectionAnchor anchor = isSource ?
                    connFigure.getSourceAnchor() : connFigure.getTargetAnchor();
            if (anchor instanceof IModelAwareAnchor) {
                ((IModelAwareAnchor) anchor).setConnectionType(isSource,
                        edgeType,
                        index, totalLength);
                conn.refresh();
            }
        }

    }

    /**
     * @generated
     */
    public class ActivityFigure extends
            org.eclipse.stp.bpmn.figures.activities.ActivityFigure {

        /**
         * @generated
         */
        public ActivityFigure() {

            this.setActivityType(((Activity) resolveSemanticElement()).
                    getActivityType().getName());
            this.setForegroundColor(org.eclipse.draw2d.ColorConstants.black);

            createContents();
        }

        /**
         * @notgenerated
         */
        private void createContents() {
            ActivityType activityType = ((Activity) getPrimaryView()
                    .getElement()).getActivityType();
            if (activityType.equals(ActivityType.TASK_LITERAL)) {
                createContentsGen();
            }
        }

        /**
         * @generated not
         */
        private void createContentsGen() {
            org.eclipse.gmf.runtime.draw2d.ui.figures.WrapLabel fig_0 = new org.eclipse.gmf.runtime.draw2d.ui.figures.WrapLabel();
            fig_0.setTextWrap(true);
            fig_0.setText("Task");
            
            setBorder(new AbstractFlowBorder() {
            	@Override
            	public int getLeftMargin() {
            		return 2;
            	}

            	@Override
            	public int getRightMargin() {
            		return 2;
            	}
            	
            	@Override
            	public Insets getInsets(IFigure figure) {
            		return new Insets(0, 2, 0, 2);
            	}
            });
            
            setFigureActivityNameFigure(fig_0);

            Object layData0 = null;

            this.add(fig_0, layData0);
        }

        /**
         * @generated
         */
        private org.eclipse.gmf.runtime.draw2d.ui.figures.WrapLabel fActivityNameFigure;

        /**
         * @generated
         */
        public org.eclipse.gmf.runtime.draw2d.ui.figures.WrapLabel getFigureActivityNameFigure() {
            return fActivityNameFigure;
        }

        /**
         * @generated
         */
        private void setFigureActivityNameFigure(
                org.eclipse.gmf.runtime.draw2d.ui.figures.WrapLabel fig) {
            fActivityNameFigure = fig;
        }

        /**
         * @generated
         */
        private boolean myUseLocalCoordinates = false;

        /**
         * @generated
         */
        protected boolean useLocalCoordinates() {
            return myUseLocalCoordinates;
        }

        /**
         * @generated
         */
        protected void setUseLocalCoordinates(boolean useLocalCoordinates) {
            myUseLocalCoordinates = useLocalCoordinates;
        }

    }

    @Override
    /**
     * @notgenerated
     */
    public EditPolicy getPrimaryDragEditPolicy() {
        return new ResizableActivityEditPolicy();
    }

    /**
     * @notgenerated
     * @param activityType
     * @return
     */
    private static int getShapeType(int activityType) {
        int shapeType;
        switch (activityType) {
        case ActivityType.TASK:
            shapeType = SHAPE_RECTANGLE;
            break;
        case ActivityType.GATEWAY_DATA_BASED_EXCLUSIVE:
        case ActivityType.GATEWAY_DATA_BASED_INCLUSIVE:
        case ActivityType.GATEWAY_EVENT_BASED_EXCLUSIVE:
        case ActivityType.GATEWAY_PARALLEL:
            shapeType = SHAPE_DIAMOND;
            break;
        default:
            shapeType = SHAPE_CIRCLE;
        }
        return shapeType;
    }


    /**
     * @notgenerated
     */
    @Override
    public Command getCommand(Request _request) {
        if (_request instanceof CreateUnspecifiedTypeConnectionRequest) {
            Object model = ((CreateUnspecifiedTypeConnectionRequest) _request)
                    .getTargetEditPart().getModel();

            if ((model instanceof Node)) {
                List elTypes = ((CreateUnspecifiedTypeConnectionRequest) _request)
                        .getElementTypes();
                MetamodelType connType = (MetamodelType) BpmnElementTypes.MessagingEdge_3002;

                for (int i = 0; i < elTypes.size(); i++) {
                    if (elTypes.get(i) instanceof MetamodelType) {
                        connType = (MetamodelType) elTypes.get(i);
                        break;
                    }
                }

                if (connType == BpmnElementTypes.MessagingEdge_3002) {
                    // message connection rules for gateways
                    MessageConnectionValidator messageValidator = new MessageConnectionValidator();

                    if (_request instanceof CreateUnspecifiedTypeConnectionRequest) {
                        CreateUnspecifiedTypeConnectionRequest request = 
                            (CreateUnspecifiedTypeConnectionRequest) _request;
                        // System.out.println("Reversed = " +
                        // request.isDirectionReversed());
                        if (request.isDirectionReversed()) {
                            // do nothing
                        } else {
                            if (request.getSourceEditPart() != null
                                    && request.getTargetEditPart() != null) {
                                for (Iterator iter = 
                                        request.getAllRequests().iterator(); iter.hasNext();) {
                                    CreateConnectionRequest connectionRequest = 
                                        (CreateConnectionRequest) iter.next();
                                    if (connectionRequest.getSourceEditPart() != null
            && connectionRequest.getTargetEditPart() != null
            && connectionRequest.getSourceEditPart() == request.getSourceEditPart()
            && connectionRequest.getTargetEditPart() == request.getTargetEditPart()) {
                                        if (!messageValidator.canConnect(
                                                request.getSourceEditPart(),
                                                request.getTargetEditPart())) {
                                            return null;
                                        }
                                    } else {
                                        // the connection is direction reversed
                                        if (!messageValidator.canConnect(
                                                request.getTargetEditPart(),
                                                request.getSourceEditPart())) {
                                            return null;
                                        }
                                    }
                                }
                            }
                            if (request.getSourceEditPart() == null
                                    && request.getTargetEditPart() != null) {
                                if (!messageValidator.canStart(request
                                        .getTargetEditPart())) {
                                    return null;
                                }
                            }
                        }
                    }
                } else if (connType == BpmnElementTypes.SequenceEdge_3001) {
                    // edge connection rules
                    EdgeConnectionValidator edgeValidator = new EdgeConnectionValidator();
                    
                    if (_request instanceof CreateConnectionRequest) {
                    	CreateConnectionRequest connectionRequest = (CreateConnectionRequest) _request;
                    	
                    	if (connectionRequest.getTargetEditPart() == null ||
                    			connectionRequest.getSourceEditPart() == null) {
                    		// close your eyes, scream and pass it to the policy
                    		// in charge of showing the diagram assistant.
                    		return super.getCommand(_request);
                    	}
                    	if (connectionRequest.getSourceEditPart() != null
			&& connectionRequest.getTargetEditPart() != null
			&& connectionRequest.getSourceEditPart() == connectionRequest.getSourceEditPart()
			&& connectionRequest.getTargetEditPart() == connectionRequest.getTargetEditPart()) {
                    		if (!edgeValidator.canConnect(
                                    connectionRequest.getSourceEditPart(),
                                    connectionRequest.getTargetEditPart())) {
                    			return null;
                    		}
                    	} else {
                    		// the connection is direction reversed
                    		if (!edgeValidator.canConnect(
                                    connectionRequest.getTargetEditPart(),
                                    connectionRequest.getSourceEditPart())) {
                    			return null;
                    		}
                    	}
                    }
                    
                    if (_request instanceof CreateUnspecifiedTypeConnectionRequest) {
                        CreateUnspecifiedTypeConnectionRequest request = (CreateUnspecifiedTypeConnectionRequest) _request;
                        // System.out.println("Reversed = " +
                        // request.isDirectionReversed());
                        if (request.isDirectionReversed()) {
                            // do nothing
                        } else {
                            if (request.getSourceEditPart() != null
                                    && request.getTargetEditPart() != null) {
                                if (request.getSourceEditPart().equals(
                                        request.getTargetEditPart())) {
                                    return null;
                                }
                                for (Iterator iter = request.getAllRequests()
                                        .iterator(); iter.hasNext();) {
                                    CreateConnectionRequest connectionRequest = 
                                        (CreateConnectionRequest) iter.next();
                                    if (connectionRequest.getSourceEditPart() != null
                                && connectionRequest.getTargetEditPart() != null
                                && connectionRequest.getSourceEditPart() == request.getSourceEditPart()
                                && connectionRequest.getTargetEditPart() == request.getTargetEditPart()) {
                                        if (!edgeValidator.canConnect(
                                                request.getSourceEditPart(),
                                                request.getTargetEditPart())) {
                                            return null;
                                        }
                                    } else {
                                        // the connection is direction reversed
                                        if (!edgeValidator.canConnect(
                                                request.getTargetEditPart(),
                                                request.getSourceEditPart())) {
                                            return null;
                                        }
                                    }
                                }
                            }

                            if (request.getSourceEditPart() == null
                            		&& request.getTargetEditPart() != null) {
                            	if (!edgeValidator.canStart(
                                        request.getTargetEditPart())) {
                            		return null;
                            	}
                            }

                            
                        }
                    }
                }
            }
        }

        return super.getCommand(_request);
    }

    /**
     * @notgenerated
     */
    public boolean isChildAdded() {
        return isChildAdded;
    }

    /**
     * @notgenerated
     */
    public void setChildAdded(boolean isChildAdded) {
        this.isChildAdded = isChildAdded;
    }

    /**
     * @notgenerated
     */
    @Override
    public DragTracker getDragTracker(Request request) {
        return new TaskDragEditPartsTrackerEx(this);
    }

    public int getHandlePosition() {
        return handlePosition;
    }

    public void setHandlePosition(int handlePosition) {
        this.handlePosition = handlePosition;
    }
}
